/*************************************************************************\
*   				Xcard 1.2				  *
*									  *
* Copyright 1991, Kenneth C. Nelson			 	          *
*									  *
* Rights       : I, Ken Nelson own Xcard.  You may compile, modify, and   *
*                use Xcard.  You may not sell Xcard or anything derived   *
*		 from Xcard in binary or source for more than reasonable  *
*                media costs.                                             *
*                                                                         *
*		 Xcard uses a dynamic object library	   	          *
*		 that I don't own.  See the subdirectory dynobj for  	  *
*		 restrictions on its use.				  *
*								          *
*                Please feel free to modify Xcard. See Todo for details.  *
*									  *
\*************************************************************************/


/*
 * editor.c - provides a generic popup editor.
 *	      with an OK and CANCEL button.
 *
*/



#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>

#include <X11/Xaw/Label.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Command.h>


#include "xmisc.h"
#include "editor.h"


static Widget  editorPopup;
static Widget  editorLabel;
static Widget  editorTextWidget;

/*
 * the action passed in by the user for when the button is pressed.
 *
*/

void (*editorButtonAction)() = NULL;



#define EDITOR_WIDTH 300
#define EDITOR_HEIGHT 300


/*
 * CBeditorOk()  - the callback for the editor OK button.
 *
*/

/*ARGSUSED*/
static void CBeditorOk(w,call_data,client_data)
  Widget     w;
  XtPointer  call_data;
  XtPointer  client_data;
{
  XtPopdown(editorPopup);
  if (editorButtonAction != NULL)
  {
     (*editorButtonAction)(editorText(),TRUE); 
  }
}



/*
 * CBeditorOk()  - the callback for the editor OK button.
 *
*/

/*ARGSUSED*/
static void CBeditorCancel(w,call_data,client_data)
  Widget     w;
  XtPointer  call_data;
  XtPointer  client_data;
{
  XtPopdown(editorPopup);
  if (editorButtonAction != NULL)
  {
     (*editorButtonAction)(editorText(),FALSE); 
  }
}




/*
 * buildPopupEditor() - builds the popup editor, attaches it to parent.
 *			if parent is NULL then attaches it to the
 *			roolevel window.
 */
 

void buildPopupEditor(parent)
  Widget parent;
{
   Widget pane,box,okButton,cancelButton;

   editorPopup = XtCreatePopupShell(
				    "Editor",
				    transientShellWidgetClass,
				    parent,
				    NULL,
				    0
				   );

   pane = XtVaCreateManagedWidget(
				  "editorPane",
				  panedWidgetClass,
				  editorPopup,
				  NULL
				 );

   editorLabel  = XtVaCreateManagedWidget(
				    "editorLabel",
				    labelWidgetClass,
				    pane,
				    XtNshowGrip,FALSE,
				    NULL
				   );

   box  = XtVaCreateManagedWidget(
				    "editorBox",
				    boxWidgetClass,
				    pane,
				    XtNshowGrip,FALSE,
				    NULL
				   );

   okButton  = XtVaCreateManagedWidget(
				    "okEditor",
				    commandWidgetClass,
				    box,
				    XtNlabel,"Okay",
				    NULL
				   );

   XtAddCallback(okButton,XtNcallback,CBeditorOk,NULL);

   cancelButton = XtVaCreateManagedWidget(
				    "cancelEditor",
				    commandWidgetClass,
				    box,
				    XtNlabel,"Cancel",
				    NULL
				   );

   XtAddCallback(cancelButton,XtNcallback,CBeditorCancel,NULL);


   editorTextWidget = XtVaCreateManagedWidget(
					      "editorTextWidget",
					      asciiTextWidgetClass,
					      pane,
					      XtNwidth,EDITOR_WIDTH,
					      XtNheight,EDITOR_HEIGHT,
					      XtNeditType,XawtextEdit,
					      NULL
					     );
					     
}





/*
 * popupEditor() - popup the popup editor, centered at where. If where
 *		   is null then it just places it where the window
 *		   manager wants it.  The name of the shell can be
 *		   replaced with editorName, the initial string to
 *		   edit is specified with string, string should be
 *		   big enough to hold what you need to edit. Action
 *		   is called when one of the buttons is pushed.
 *		   The format for action is:
 *		     void action(string,status) status is TRUE
 *		   if okay was pushed, FALSE if CANCEL was pushed.
 *
 *
*/


void popupEditor(where,editorName,string,action)
  Widget   where;
  char    *editorName;
  char    *string;
  void    (*action)();
{
  Position x,y;
  
  if (where != NULL)
  {
    centerCoords(where,&x,&y);
    XtVaSetValues(editorPopup,
		  XtNx,x,
		  XtNy,y,
		  NULL
		 );
  }

  editorButtonAction = action;

  XtVaSetValues(
		 editorLabel,
		 XtNlabel,editorName,
		 NULL
	       );

  XtVaSetValues(
		 editorTextWidget,
		 XtNstring,string,
		 NULL
	       );

 XtPopup(editorPopup,XtGrabExclusive);

}




/*
 * editorText() - returns the text in the textEditor.
 *
*/

char *editorText()
{
  char *text;

  XtVaGetValues(editorTextWidget,"string",&text,NULL);
  return text;
}




