/*************************************************************************\
*   				Xcard 1.2				  *
*									  *
* Copyright 1991, Kenneth C. Nelson			 	          *
*									  *
* Rights       : I, Ken Nelson own Xcard.  You may compile, modify, and   *
*                use Xcard.  You may not sell Xcard or anything derived   *
*		 from Xcard in binary or source for more than reasonable  *
*                media costs.                                             *
*                                                                         *
*		 Xcard uses a dynamic object library	   	          *
*		 that I don't own.  See the subdirectory dynobj for  	  *
*		 restrictions on its use.				  *
*								          *
*                Please feel free to modify Xcard. See Todo for details.  *
*									  *
\*************************************************************************/


/*
 * headers.c - provides routines to operate and manage the header
 *	       list area of XCard.  The list is a Text widget
 *	       modified to do what the list and viewport widget should
 *	       do.
 *
 */


#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Viewport.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/AsciiText.h>



#include "cards.h"
#include "headers.h"
#include "xmisc.h"


/*
 * The header list widget. This is local to this file, and only
 * routines in this file should make any changes to it.
 *
 */
 
static Widget headerList;


/*
 * returns the number of carriage returns in a string up the ending
 * position passed in end.
 *
 */
 
static int countReturns(string,end)
  char *string;
  int  end;
{
  int pos = 0;
  int count = 0;

  if (string != NULL)
  {
    while (pos <= end && string[pos] != '\0')
    {
      if (string[pos] == '\n')
      {
        count++;
      }
      pos++;
    }
  }
  return count;
}


/*
 * returns the character position of the first character of
 * line number line num. Returns 0 if bad linenumber.
 *
 */

static int gotoLine(string,lineNum)
  char *string;
  int  lineNum;
{
  int pos=0;
  int currLine = 1;
  
  while (string != NULL && pos <= strlen(string) && currLine < lineNum)
  {
    pos++;
    if (string[pos] == '\n')
    {
      currLine++;
    }
  }
  if (currLine != lineNum)
    return 0;
  else
    return pos+1;
}



/*
 * highlights a line in the header list editor widget.
 *
 */


static void highlightLine(lineNum)
{

   int pos;
   char *text;
   int start,end;   

   /*
    * find the start of the line
    *
    */

    XtVaGetValues(headerList,"string",&text,NULL);

    if (text != NULL)
    {
      start = gotoLine(text,lineNum);
      end = start+1;
      while (text[end] != '\0' && text[end] != '\n')
      {
        end++;
      }
      XawTextUnsetSelection(headerList);
      XawTextSetSelection(headerList,
		          (XawTextPosition) start,
		          (XawTextPosition) end
		         );
      XawTextSetInsertionPoint(headerList,(XawTextPosition) start);
            
    }
}

 

/*
 * ACTheaderSelect() - the action when one of the list headers is
 *		      selected.
 */
 
void ACTheaderSelect(w,event)
  Widget w;
  XButtonEvent  event;
{

   int num;
   XawTextPosition start,end;
   char *numstr;
     
   /*
    * Save the current text in the editor into the card
    * it was editing.
    *
    */

    saveCurrCard();


    /*
     * Get the text of the current selection, convert it
     * into a number and use it.
     *
     */

    XawTextGetSelectionPos(headerList,&start,&end);

    XtVaGetValues(headerList,"string",&numstr,NULL);
    
    num = countReturns(numstr,start);  /* Count returns up to start */

    setCurrentCard(num);

    setHeaderArea(headerCurrentCard());
    setCardEditorText(bodyCurrentCard());
    
}


/*
 * addCardHeaderList() - adds the widgets which provide the header of cards list.
 *			 The list is built up of a viewport and a list widget.
 *
 */

void addCardHeaderListArea(parent)
	Widget parent;
{

  static XtActionsRec headerActions[] = {
					  {"headerSelect",ACTheaderSelect},
					  {"nop",ACTnop}
				        };
				      
  XtTranslations headerTranslations;

  static XawTextSelectType selectionArray[] = {
					       XawselectLine,
					       XawselectNull
					     };


  static char headerTransStr[] = "#override\n\
			<Btn1Down>   : select-start() extend-adjust() headerSelect()\n\
			<Btn1Motion> : nop() \n \
			<Btn2Down>   : select-start() extend-adjust() headerSelect()\n\
			<Btn2Motion> : nop() \n \
			<Btn3Down>   : select-start() extend-adjust() headerSelect()\n\
			<Btn3Motion> : nop() \n \
			";
			
  headerList = XtVaCreateManagedWidget(
				       "headerList",
				       asciiTextWidgetClass,
				       parent,
				       XtNeditType,XawtextRead,
				       XtNheight,200,
				       XtNselectTypes,selectionArray,
				       XtNscrollVertical,XawtextScrollAlways,
				       NULL
				      );

  
  XtAddActions(headerActions,XtNumber(headerActions));

  headerTranslations = XtParseTranslationTable(headerTransStr);
  XtOverrideTranslations(headerList,headerTranslations);
 
}



/*
 * highlightHeader() - highlights the header and sets the
 *		       body editor to the headers body.
 *		       Changes current card position. 
 */

void highlightHeader(num)
 int num;
{

  highlightLine(num+1);
  setCurrentCard(num);
  setHeaderArea(headerCurrentCard());
  setCardEditorText(bodyCurrentCard());
  
}



/*
 * updateCardHeaderList() - looks at the current card file, builds
 *			    a new list and then updates the list widget.
 *
 */

char headersList[MAX_NUM_CARDS*MAX_CARD_LINE];

void  updateCardHeaderList()
{
  int pos=0;
  char temp[100];
  
  if (setCurrentCard(FIRSTCARD))
  {
    pos = 0;
    do
    {
      if (pos == 0)
      {
        strcpy(headersList,headerCurrentCard());
      }
      else
      {
        strcat(headersList,headerCurrentCard());
      }
      strcat(headersList,"\n");      
      pos++;
    }
    while (setCurrentCard(NEXTCARD));
  }

  XtVaSetValues(headerList,
		XtNstring,headersList,
		NULL
	       );
	       
}


/*
 * addCardToCardHeaderList() - adds the card to the card header list
 *
 */

void addCardToCardHeaderList(card)
  Card card;
{
}



/*
 * currentCardOfCardHeaderList() - returns the current selected Card from the
 *				   card header list, or Empty_Card if none selected.
 *
 */

Card currentCardOfCardHeaderList()
{
}




/*
 * rmCardFromCardHeaderList() - removes the specified card from the card header
 *				list. If it is not there then nothing happens.
 *
 */

void rmCardFromCardHeaderList()
{
}


